{%  extends "openassessmentblock/peer/oa_peer_assessment.html" %}
{% load i18n %}

{% block list_item %}
<li id="openassessment__peer-assessment__{{ xblock_id }}" class="openassessment__steps__step step--peer-assessment is--showing is--complete ui-slidable__container" tabindex="-1">
{% endblock %}

{% block button %}
    <button class="ui-slidable" aria-expanded="true" id="oa_peer_{{ xblock_id }}" aria-controls="oa_peer_{{ xblock_id }}_content" aria-labeledby="oa_step_title_peer">
        <span class="icon fa fa-caret-right" aria-hidden="false"/>
    </button>
{% endblock %}

{% block title %}
<span class="step__status">
  <span id="oa_step_status_peer" class="step__status__value">
    <span class="icon fa fa-check" aria-hidden="true"></span>
    <span class="copy">
      {% with graded_string=graded|stringformat:"s" %}
      {% blocktrans with num_graded='<span class="step__status__value--completed">'|safe|add:graded_string|add:'</span>'|safe %}
        Complete ({{ num_graded }})
      {% endblocktrans %}
      {% endwith %}
    </span>
  </span>
</span>
{% endblock %}

{% block body %}
<div role="region" class="ui-slidable__content" aria-labelledby="oa_peer_{{ xblock_id }}" id="oa_peer_{{ xblock_id }}_content">
    <div class="wrapper--step__content">

        <div class="step__message message message--complete">
            <h5 class="message__title">{% trans "Status" %}</h5>
            <div class="message__content">
              <p>{% trans "You have successfully completed all of the required peer assessments for this assignment. You may assess additional peer responses if you want to. Completing additional assessments will not affect your final grade." %}</p>

              <p><strong>{% trans "All submitted peer responses have been assessed. Check back later to see if more learners have submitted responses." %}</strong></p>
            </div>
        </div>
    </div>
</div>
{% endblock %}
