{% load i18n %}
{% spaceless %}
<li id="openassessment__leaderboard__{{ xblock_id }}" class="openassessment__steps__step step--leaderboard is--initially--collapsed ui-slidable__container" tabindex="-1">
    <header class="step__header ui-slidable__control">
        <h4 class="step__title">
            <button class="ui-slidable" aria-expanded="false" id="oa_leaderboard_{{ xblock_id }}" aria-controls="oa_leaderboard_{{ xblock_id }}_content" aria-describedby="oa_step_status oa_step_deadline">
                <span class="icon fa fa-caret-right" aria-hidden="false"/>
            </button>
            <span class="wrapper--copy">
                <span class="step__label">{% trans "Top Responses" %} </span>
            </span>
        </h4>
    </header>

    <div class="ui-slidable__content"  aria-labelledby="oa_leaderboard_{{ xblock_id }}" id="oa_leaderboard_{{ xblock_id }}_content">
        <div class="wrapper--step__content">
            <div class="step__content">
                <div class="leaderboard__description">
                    <p>{% trans "After you complete all the steps of this assignment, you can see the top-scoring responses from your peers." %}</p>
                </div>
            </div>
        </div>
    </div>
</li>
{% endspaceless %}
