# Generated by Django 2.2.24 on 2021-11-17 21:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('assessment', '0006_TeamWorkflows'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staffworkflow',
            name='assessment',
            field=models.CharField(blank=True, db_index=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='staffworkflow',
            name='cancelled_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='staffworkflow',
            name='grading_completed_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='staffworkflow',
            name='grading_started_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='staffworkflow',
            name='scorer_id',
            field=models.CharField(blank=True, db_index=True, max_length=40),
        ),
    ]
