{% load i18n %}
{% spaceless %}
<li class="openassessment_assessment_module_settings_editor" id="oa_staff_assessment_editor">
    <div class="drag-handle action"></div>
    <div class="openassessment_inclusion_wrapper">
        <input type="checkbox" id="include_staff_assessment"
            {% if assessments.staff_assessment.required %} checked="true" {% endif %}>
        <label for="include_staff_assessment">{% trans "Step: Staff Assessment" %}</label>
    </div>
    <div class="openassessment_assessment_module_editor">
        <p class="openassessment_description">
            {% trans "Staff Assessment gates sets the final grade for students if enabled with other steps, though it can also be used as a standalone step." %}
        </p>
    </div>
</li>
{% endspaceless %}
