{% spaceless %}
{% load i18n %}
{% load oa_extras %}
<ol class="submission__answer__display__content">
    {% for part in answer.parts %}
        <li class="submission__answer__part">
            <div class="submission__answer__part__text">
                <h5 class="submission__answer__part__text__title">{% trans "The question for this section" %}</h5>
            </div>
            <article class="submission__answer__part__prompt">
                <div class="submission__answer__part__prompt__value">
                    {% if prompts_type == 'html' %}
                        {{ part.prompt.description|safe }}
                    {% else %}
                        {{ part.prompt.description|linebreaks }}
                    {% endif %}
                </div>
            </article>
            {% if part.text %}
            <div class="submission__answer__part__text">
                <h5 class="submission__answer__part__text__title">{{ answer_text_label }}</h5>
                {% if text_response_editor == 'tinymce' %}
                    <textarea class="submission__answer__part__text__value" disabled>
                        {{ part.text|escape }}
                    </textarea>
                {% else %}
                    <div class="submission__answer__part__text__value">
                        {{ part.text|escape|link_and_linebreak }}
                    </div>
                {% endif %}
            </div>
            {% endif %}
        </li>
    {% endfor %}
</ol>
{% endspaceless %}
