{% load i18n %}
{% spaceless %}
<li class="openassessment_assessment_module_settings_editor" id="oa_self_assessment_editor">
    <div class="drag-handle action"></div>
    <div class="openassessment_inclusion_wrapper">
        <input id="include_self_assessment" type="checkbox"
            {% if assessments.self_assessment %} checked="true" {% endif %}>
        <label for="include_self_assessment">{% trans "Step: Self Assessment" %}</label>
    </div>
    <div class="openassessment_assessment_module_editor">
        <p class="openassessment_description">
            {% trans "Self Assessment asks learners to grade their own submissions against the rubric." %}
        </p>
    </div>
</li>
{% endspaceless %}
