{% extends "openassessmentblock/response/oa_response.html" %}
{% load i18n %}

{% block list_item %}
<li id="openassessment__response__{{ xblock_id }}"
    class="openassessment__steps__step step--response is--complete ui-slidable__container {% if allow_latex %}allow--latex{%endif%} is--initially--collapsed"
    tabindex="-1">
{% endblock %}

{% block button %}
    <button class="ui-slidable" aria-expanded="true" id="oa_response_{{ xblock_id }}" aria-controls="oa_response_{{ xblock_id }}_content" aria-labeledby="oa_step_title_response">
        <span class="icon fa fa-caret-right" aria-hidden="false"/>
    </button>
{% endblock %}

{% block title %}
<span class="step__status">
  <span id="oa_step_status_response" class="step__status__value">
    <span class="icon fa fa-check" aria-hidden="true"></span>
    <span class="copy">{% trans "Complete" %}</span>
  </span>
</span>
{% endblock %}

{% block body %}
<div class="ui-slidable__content" aria-labelledby="oa_response_{{ xblock_id }}" id="oa_response_{{ xblock_id }}_content">
    <div class="wrapper--step__content">

        <div class="step__message message message--complete">
            <h5 class="message__title">{% trans "Status" %}</h5>
            <div class="message__content">
                {% trans "Your response has been submitted. You will receive your grade after all steps are complete and your response is fully assessed." %}
                {% if peer_incomplete and self_incomplete %}
                    {% blocktrans with peer_start_tag='<a data-behavior="ui-scroll" href="#openassessment__peer-assessment__'|add:xblock_id|add:'">'|safe self_start_tag='<a data-behavior="ui-scroll" href="#openassessment__self-assessment__'|add:xblock_id|add:'">'|safe end_tag='</a>'|safe %}
                        You still need to complete the {{ peer_start_tag }}peer assessment{{ end_tag }} and {{ self_start_tag }}self assessment{{ end_tag }} steps.
                    {% endblocktrans %}
                {% elif peer_incomplete %}
                    {% blocktrans with start_tag='<a data-behavior="ui-scroll" href="#openassessment__peer-assessment__'|add:xblock_id|add:'">'|safe end_tag='</a>'|safe%}
                        You still need to complete the {{ start_tag }}peer assessment{{ end_tag }} step.
                    {% endblocktrans %}
                {% elif self_incomplete %}
                    {% blocktrans with start_tag='<a data-behavior="ui-scroll" href="#openassessment__self-assessment__'|add:xblock_id|add:'">'|safe end_tag='</a>'|safe%}
                        You still need to complete the {{ start_tag }}self assessment{{ end_tag }} step.
                    {% endblocktrans %}
                {% endif %}
            </div>
        </div>

        <div class="step__content">
            <article class="submission__answer__display">
                {% trans "Your response" as translated_label %}
                {% include "openassessmentblock/oa_submission_answer.html" with answer=student_submission.answer answer_text_label=translated_label %}

                {% trans "Your Uploaded Files"  as translated_header %}
                {% include "openassessmentblock/oa_uploaded_file.html" with file_upload_type=file_upload_type file_urls=file_urls header=translated_header class_prefix="submission__answer" including_template="response_submitted" xblock_id=xblock_id %}

                {% include "openassessmentblock/oa_team_uploaded_files.html" with file_upload_type=file_upload_type team_file_urls=team_file_urls header=translated_header class_prefix="submission__team__answer" including_template="response_submitted" xblock_id=xblock_id %}
            </article>
        </div>
    </div>
</div>
{% endblock %}
